package com.example.softapdemo;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.SocketException;
import java.util.Arrays;
import java.util.Formatter;

import android.annotation.SuppressLint;
import android.content.Context;
import android.net.ConnectivityManager;
import android.net.Network;
import android.net.NetworkCapabilities;
import android.net.NetworkRequest;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;

public class SoftApConfig {
	/**
	 * single instance UdpClient
	 * */
	private static SoftApConfig mSocketClient = null;
	ConnectivityManager connectivityManager;
	private SoftApConfig() {

	}

	public static SoftApConfig getInstance() {
		if (mSocketClient == null) {
			synchronized (SoftApConfig.class) {
				mSocketClient = new SoftApConfig();
			}
		}
		return mSocketClient;
	}

	private DatagramSocket socket = null;
	private InetAddress serverAddress = null;
	private SocketThread mSocketThread;
	private boolean isStart = false;// thread flag
	private String pwd;
	private String ssid;

	public void startConfig(Context mContext,String ssid, String pwd) {
		this.pwd = pwd;
		this.ssid = ssid;
		connectivityManager = (ConnectivityManager)mContext.getSystemService(Context.CONNECTIVITY_SERVICE);
        NetworkRequest.Builder builder;
		 if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
	            builder = new NetworkRequest.Builder();
	            builder.addCapability(NetworkCapabilities.NET_CAPABILITY_INTERNET);
	            builder.addTransportType(NetworkCapabilities.TRANSPORT_WIFI);
	            connectivityManager.requestNetwork(builder.build(), new ConnectivityManager.NetworkCallback() {
	                @Override
	                public void onAvailable(Network network) {
	                    Log.e("netId",network.toString()+" ");
	                    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
	                        connectivityManager.bindProcessToNetwork(network);
	                    } else {
	                        connectivityManager.setProcessDefaultNetwork(network);
	                    }
	                    try {
	            			socket = new DatagramSocket(12345);
	            			serverAddress = InetAddress.getByName("10.10.100.254");
	            		} catch (Exception e) {
	            			e.printStackTrace();
	            		}
	            		 if (!isStart) {
	             			isStart = true;
	             			mSocketThread = new SocketThread();
	             			mSocketThread.start();
	             		}
	            		
	                    connectivityManager.unregisterNetworkCallback(this);
	                }
	            });
	        }
	}
	
	public void stopConfig()
	{
		isStart=false;
		mSocketThread = null;
		connectivityManager.bindProcessToNetwork(null);
	}

	class SocketThread extends Thread {
		@Override
		public void run() {
			super.run();
			while (isStart) {
				try {
					byte data[] = getSendData();
					Log.e("配网指令", bytesToHexString(data, " "));
					DatagramPacket packet = new DatagramPacket(data,
							data.length, serverAddress, 12414); // ③
					socket.send(packet);
				} catch (Exception e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				try {
					sleep(5000);
				} catch (InterruptedException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}

			}
		}
	}

	private byte[] getSendData() {
		byte[] pwdByte = pwd.getBytes();
		byte[] ssidByte = ssid.getBytes();
		byte[] data = new byte[pwdByte.length + ssidByte.length + 12];
		int offset = 0;
		data[offset++] = 0;
		data[offset++] = 0;
		data[offset++] = 0;
		data[offset++] = 3;

		// length
		data[offset++] = (byte) ((pwdByte.length + ssidByte.length + 7) & 0xFF);

		data[offset++] = 0;
		data[offset++] = 0;
		data[offset++] = 1;
		data[offset++] = 0;
		data[offset++] = (byte) (ssidByte.length & 0xFF);
		System.arraycopy(ssidByte, 0, data, offset, ssidByte.length);
		offset = offset+ssidByte.length;
		data[offset++] = 0;
		data[offset++] = (byte) (pwdByte.length & 0xFF);
		System.arraycopy(pwdByte, 0, data, offset, pwdByte.length);

		return data;
	}
	
	/**
	 * byte数组转成十六进制字符串
	 * 
	 * @param array
	 *            原数组
	 * @param separator
	 *            分隔符
	 * @return
	 */
	public static String bytesToHexString(byte[] array, String separator) {

		if (array == null || array.length == 0)
			return "";

		StringBuilder sb = new StringBuilder();

		Formatter formatter = new Formatter(sb);
		formatter.format("%02X", array[0]);

		for (int i = 1; i < array.length; i++) {

			if (separator != null)
				sb.append(separator);

			formatter.format("%02X", array[i]);
		}

		formatter.flush();
		formatter.close();

		return sb.toString();
	}

}
